@@ -1,4 +1,4 @@
-# Copyright 2020 Oscar Higgott
+# Copyright 2022 PyMatching Contributors
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from pymatching._cpp_mwpm import (randomize, set_seed, rand_float, BlossomFailureException)
-from pymatching.matching import *
+from pymatching._cpp_pymatching import (randomize, set_seed, rand_float)  # noqa
+from pymatching._cpp_pymatching import main as cli  # noqa
+from pymatching.matching import Matching  # noqa
 
-randomize() # Set random seed using std::random_device
+randomize()  # Set random seed using std::random_device
