@@ -23,7 +23,7 @@ from scipy.sparse import csc_matrix, spmatrix
 import matplotlib.cbook
 
 if TYPE_CHECKING:
-    import stim   # pragma: no cover
+    import stim  # pragma: no cover
 
 import pymatching._cpp_pymatching as _cpp_pm
 
@@ -323,13 +323,111 @@ class Matching:
         else:
             return correction
 
-    def decode_to_matched_dets_array(self,
-                                     syndrome: Union[np.ndarray, List[int]]
-                                     ) -> Union[np.ndarray, Tuple[np.ndarray, int]]:
+    def decode_batch(
+            self,
+            shots: np.ndarray,
+            *,
+            return_weights: bool = False,
+            bit_packed_shots: bool = False,
+            bit_packed_predictions: bool = False) -> Union[np.ndarray, Tuple[np.ndarray, np.ndarray]]:
         """
-        Decode the syndrome `syndrome` using minimum-weight perfect matching, returning the pairs of
-        matched detection events (or detection events matched to the boundary) as a 2D numpy array. Note that
-        (unlike `Matching.decode`), this method currently only supports non-negative edge weights.
+        Decode from a 2D `shots` array containing a batch of syndrome measurements. A faster
+        alternative to using `pymatching.Matching.decode` and iterating over the shots in Python.
+
+        Parameters
+        ----------
+        shots : np.ndarray
+            A 2D numpy array of shots to decode, of `dtype=np.uint8`.
+
+            If `bit_packed_shots==False`, then
+            `shots` should have shape `shots.shape=(num_shots, syndrome_length)`, where `num_shots` is the
+            number of shots (samples), and `syndrome_length` is the length of the binary syndrome vector to be
+            decoded for each shot. If `len(self.boundary)==0` (e.g. if there is no boundary, or only a virtual
+            boundary node, the default when loading from stim) then `syndrome_length=self.num_detectors`.
+            However, `syndrome_length` is permitted to be as high as `self.num_nodes` in case the graph contains
+            detectors nodes with an index larger than `self.num_detectors-1` (when `len(self.boundary)>0`).
+
+            If `bit_packed_shots==True` then `shots` should have shape
+            `shots.shape=(num_shots, math.ceil(syndrome_length / 8))`. Bit packing should be done using little endian
+            order on the last axis (like ``np.packbits(data, bitorder='little', axis=1)``), so that the bit for
+            detection event `m` in shot `s` can be found at ``(dets[s, m // 8] >> (m % 8)) & 1``.
+        return_weights : bool
+            If True, then also return a numpy array containing the weights of the solutions for all the shots.
+            By default, False.
+        bit_packed_shots : bool
+            Set to `True` to provide `shots` as a bit-packed array, such that the bit for
+            detection event `m` in shot `s` can be found at ``(dets[s, m // 8] >> (m % 8)) & 1``.
+        bit_packed_predictions : bool
+            Set to `True` if the returned predictions should be bit-packed, with the bit for fault id `m` in
+            shot `s` in ``(obs[s, m // 8] >> (m % 8)) & 1``
+
+        Returns
+        -------
+        predictions: np.ndarray
+            The batch of predictions output by the decoder, a binary numpy array of `dtype=np.uint8` and with shape
+            `predictions.shape=(num_shots, self.num_fault_ids)`. `predictions[i, j]=1` iff the decoder predicts that
+            fault id `j` was flipped in the shot `i`.
+        weights: np.ndarray
+            The weights of the MWPM solutions, a numpy array of `dtype=float`. `weights[i]` is the weight of the
+            MWPM solution in shot `i`.
+
+        Examples
+        --------
+        >>> import pymatching
+        >>> import stim
+        >>> circuit = stim.Circuit.generated("surface_code:rotated_memory_x",
+        ...                                  distance=5,
+        ...                                  rounds=5,
+        ...                                  after_clifford_depolarization=0.005)
+        >>> model = circuit.detector_error_model(decompose_errors=True)
+        >>> matching = pymatching.Matching.from_detector_error_model(model)
+        >>> sampler = circuit.compile_detector_sampler()
+        >>> syndrome, actual_observables = sampler.sample(shots=10000, separate_observables=True)
+        >>> syndrome.shape
+        (10000, 120)
+        >>> actual_observables.shape
+        (10000, 1)
+        >>> predicted_observables = matching.decode_batch(syndrome)
+        >>> predicted_observables.shape
+        (10000, 1)
+        >>> num_errors = np.sum(np.any(predicted_observables != actual_observables, axis=1))
+
+        We can also decode bit-packed shots, and return bit-packed predictions:
+        >>> import pymatching
+        >>> import stim
+        >>> circuit = stim.Circuit.generated("surface_code:rotated_memory_x",
+        ...                                  distance=5,
+        ...                                  rounds=5,
+        ...                                  after_clifford_depolarization=0.005)
+        >>> model = circuit.detector_error_model(decompose_errors=True)
+        >>> matching = pymatching.Matching.from_detector_error_model(model)
+        >>> sampler = circuit.compile_detector_sampler()
+        >>> syndrome, actual_observables = sampler.sample(shots=10000, separate_observables=True, bit_packed=True)
+        >>> syndrome.shape
+        (10000, 15)
+        >>> actual_observables.shape
+        (10000, 1)
+        >>> predicted_observables = matching.decode_batch(syndrome, bit_packed_shots=True, bit_packed_predictions=True)
+        >>> predicted_observables.shape
+        (10000, 1)
+        >>> num_errors = np.sum(np.any(predicted_observables != actual_observables, axis=1))
+        """
+        predictions, weights = self._matching_graph.decode_batch(
+            shots,
+            bit_packed_predictions=bit_packed_predictions,
+            bit_packed_shots=bit_packed_shots
+        )
+        if return_weights:
+            return predictions, weights
+        else:
+            return predictions
+
+    def decode_to_edges_array(self,
+                              syndrome: Union[np.ndarray, List[int]]
+                              ) -> np.ndarray:
+        """
+        Decode the syndrome `syndrome` using minimum-weight perfect matching, returning the edges in the
+        solution, given as pairs of detector node indices in a numpy array.
 
         Parameters
         ----------
@@ -345,11 +443,12 @@ class Matching:
         Returns
         -------
         numpy.ndarray
-            An 2D array `pairs` giving the endpoints of the paths between detection events in the solution of the matching.
-            If there are `num_paths` paths then the shape of `pairs` is `num_paths.shape=(num_paths, 2)`, and path `i`
-            starts at detection event `pairs[i,0]` and ends at detection event `pairs[i,1]`. For a path `i` connecting
-            a detection event to the boundary (either a boundary node or the virtual boundary node), then `pairs[i,0]` is
-            is the index of the detection event, and `pairs[i,1]=-1` denotes the boundary.
+            A 2D array `edges` giving the edges in the matching solution as pairs of detector nodes (or as a detector
+            node and the boundary, for a boundary edge). If there are `num_predicted_edges` edges then the shape of
+            `edges` is `edges.shape=(num_predicted_edges, 2)`, and edge `i` is between detector node `edges[i, 0]`
+            and detector node `edges[i, 1]`. For a boundary edge `i` between a detector node `k` and the boundary
+            (either a boundary node or the virtual boundary node), then `pairs[i,0]` is `k`, and `pairs[i,1]=-1`
+            denotes the boundary (the boundary is always denoted by -1 and is always in the second column).
 
         Examples
         --------
@@ -360,10 +459,65 @@ class Matching:
         >>> m.add_edge(1, 2)
         >>> m.add_edge(2, 3)
         >>> m.add_edge(3, 4)
-        >>> matched_dets = m.decode_to_matched_dets_array([1, 0, 0, 1, 1])
+        >>> m.add_edge(4, 5)
+        >>> m.add_edge(5, 6)
+        >>> edges = m.decode_to_edges_array([0, 1, 0, 0, 1, 0, 1])
+        >>> print(edges)
+        [[ 0  1]
+         [ 0 -1]
+         [ 5  4]
+         [ 5  6]]
+        """
+        detection_events = self._syndrome_array_to_detection_events(syndrome)
+        return self._matching_graph.decode_to_edges_array(detection_events)
+
+    def decode_to_matched_dets_array(self,
+                                     syndrome: Union[np.ndarray, List[int]]
+                                     ) -> np.ndarray:
+        """
+        Decode the syndrome `syndrome` using minimum-weight perfect matching, returning the pairs of
+        matched detection events (or detection events matched to the boundary) as a 2D numpy array.
+        Each pair of matched detection events returned by this method corresponds to a shortest path
+        between the detection events in the solution to the problem: if you instead want the set of
+        all edges in the solution (pairs of detector nodes), use `Matching.decode_to_edges` instead.
+        Note that, unlike `Matching.decode`, `Matching.decode_batch` and `Matching.decode_to_edges_array`,
+        this method currently only supports non-negative edge weights.
+
+        Parameters
+        ----------
+        syndrome : numpy.ndarray
+            A binary syndrome vector to decode. The number of elements in
+            `syndrome` should equal the number of nodes in the matching graph. If
+            `syndrome` is a 1D array, then `syndrome[i]` is the syndrome at node `i` of
+            the matching graph. If `syndrome` is 2D then `syndrome[i,j]` is the difference
+            (modulo 2) between the (noisy) measurement of stabiliser `i` in time
+            step `j+1` and time step `j` (for the case where the matching graph is
+            constructed from a check matrix with `repetitions>1`).
+
+        Returns
+        -------
+        numpy.ndarray
+            An 2D array `pairs` giving the endpoints of the paths between detection events in the solution of the
+            matching. If there are `num_paths` paths then the shape of `pairs` is `pairs.shape=(num_paths, 2)`, and
+            path `i` starts at detection event `pairs[i,0]` and ends at detection event `pairs[i,1]`. For a path `i`
+            connecting a detection event to the boundary (either a boundary node or the virtual boundary node), then
+            `pairs[i,0]` is the index of the detection event, and `pairs[i,1]=-1` denotes the boundary.
+
+        Examples
+        --------
+        >>> import pymatching
+        >>> m = pymatching.Matching()
+        >>> m.add_boundary_edge(0)
+        >>> m.add_edge(0, 1)
+        >>> m.add_edge(1, 2)
+        >>> m.add_edge(2, 3)
+        >>> m.add_edge(3, 4)
+        >>> m.add_edge(4, 5)
+        >>> m.add_edge(5, 6)
+        >>> matched_dets = m.decode_to_matched_dets_array([0, 1, 0, 0, 1, 0, 1])
         >>> print(matched_dets)
-        [[ 0 -1]
-         [ 3  4]]
+        [[ 1 -1]
+         [ 4  6]]
         """
         detection_events = self._syndrome_array_to_detection_events(syndrome)
         return self._matching_graph.decode_to_matched_detection_events_array(detection_events)
@@ -416,10 +570,11 @@ class Matching:
 
     def draw(self) -> None:
         """Draw the matching graph using matplotlib
-        Draws the matching graph as a matplotlib graph. Stabiliser nodes are
+        Draws the matching graph as a matplotlib graph. Detector nodes are
         filled grey and boundary nodes are filled white. The line thickness of each
         edge is determined from its weight (with min and max thicknesses of 0.2 pts
         and 2 pts respectively).
+        Each node is labelled with its id/index, and each edge is labelled with its `fault_ids`.
         Note that you may need to call `plt.figure()` before and `plt.show()` after calling
         this function.
         """
@@ -1055,18 +1210,120 @@ class Matching:
 
         Parameters
         ----------
-        model
+        model : stim.DetectorErrorModel
+            A stim DetectorErrorModel, with all error mechanisms either graphlike, or decomposed into graphlike
+            error mechanisms
 
         Returns
         -------
         pymatching.Matching
-            A `pymatching.Matching` object representing the edge-like fault mechanisms in `model`
+            A `pymatching.Matching` object representing the graphlike error mechanisms in `model`
 
+        Examples
+        --------
+        >>> import stim
+        >>> import pymatching
+        >>> circuit = stim.Circuit.generated("surface_code:rotated_memory_x",
+        ...                                  distance=5,
+        ...                                  rounds=5,
+        ...                                  after_clifford_depolarization=0.005)
+        >>> model = circuit.detector_error_model(decompose_errors=True)
+        >>> matching = pymatching.Matching.from_detector_error_model(model)
+        >>> matching
+        <pymatching.Matching object with 120 detectors, 0 boundary nodes, and 502 edges>
         """
         m = Matching()
         m._load_from_detector_error_model(model)
         return m
 
+    @staticmethod
+    def from_detector_error_model_file(dem_path: str) -> 'pymatching.Matching':
+        """
+        Construct a `pymatching.Matching` by loading from a stim DetectorErrorModel file path.
+
+        Parameters
+        ----------
+        dem_path : str
+            The path of the detector error model file
+
+        Returns
+        -------
+        pymatching.Matching
+            A `pymatching.Matching` object representing the graphlike error mechanisms in the stim DetectorErrorModel
+            in the file `dem_path`
+        """
+        m = Matching()
+        m._matching_graph = _cpp_pm.detector_error_model_file_to_matching_graph(dem_path)
+        return m
+
+    @staticmethod
+    def from_stim_circuit(circuit: 'stim.Circuit') -> 'pymatching.Matching':
+        """
+        Constructs a `pymatching.Matching` object by loading from a `stim.Circuit`
+
+        Parameters
+        ----------
+        circuit : stim.Circuit
+            A stim circuit containing error mechanisms that are all either graphlike, or decomposable into
+            graphlike error mechanisms
+
+        Returns
+        -------
+        pymatching.Matching
+            A `pymatching.Matching` object representing the graphlike error mechanisms in `circuit`, with any hyperedge
+            error mechanisms decomposed into graphlike error mechanisms. Parallel edges are merged using
+            `merge_strategy="independent"`.
+
+
+        Examples
+        --------
+        >>> import stim
+        >>> import pymatching
+        >>> circuit = stim.Circuit.generated("surface_code:rotated_memory_x",
+        ...                                  distance=5,
+        ...                                  rounds=5,
+        ...                                  after_clifford_depolarization=0.005)
+        >>> matching = pymatching.Matching.from_stim_circuit(circuit)
+        >>> matching
+        <pymatching.Matching object with 120 detectors, 0 boundary nodes, and 502 edges>
+        """
+        try:
+            import stim
+        except ImportError:  # pragma no cover
+            raise TypeError(
+                f"`circuit` must be a `stim.Circuit. Instead, got: {type(circuit)}.`"
+                "The 'stim' package also isn't installed and is required for this method. \n"
+                "To install stim using pip, run `pip install stim`."
+            )
+        if not isinstance(circuit, stim.Circuit):
+            raise TypeError(f"`circuit` must be a `stim.Circuit`. Instead, got {type(circuit)}")
+        m = Matching()
+        m._matching_graph = _cpp_pm.detector_error_model_to_matching_graph(
+            str(circuit.detector_error_model(decompose_errors=True))
+        )
+        return m
+
+    @staticmethod
+    def from_stim_circuit_file(stim_circuit_path: str) -> 'pymatching.Matching':
+        """
+        Construct a `pymatching.Matching` by loading from a stim circuit file path.
+
+        Parameters
+        ----------
+        stim_circuit_path : str
+            The path of the stim circuit file
+
+        Returns
+        -------
+        pymatching.Matching
+            A `pymatching.Matching` object representing the graphlike error mechanisms in the stim circuit
+            in the file `stim_circuit_path`, with any hyperedge error mechanisms decomposed into graphlike error
+            mechanisms. Parallel edges are merged using `merge_strategy="independent"`.
+        """
+        m = Matching()
+        m._matching_graph = _cpp_pm.stim_circuit_file_to_matching_graph(stim_circuit_path)
+        return m
+
     def _load_from_detector_error_model(self, model: 'stim.DetectorErrorModel') -> None:
         try:
             import stim
